
/* =============== GRID ELEMENTS ==================== */
.content-body {
  max-width: 1080px;
  margin: 0 auto;
  padding: 4em 0;
}
.wrapper {
  max-width: 1100px;
  width: 80%;
  margin: 0 auto;
  position: relative;
}
.flex-row {
  border: none!important;
  border-collapse: collapse;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  display: -webkit-box;
  display: -moz-box;
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex;
  clear: both;
}
.row:after,.section:after {
  clear: both;
  display: table;
  content: '';
}
.test [class*="col"]:nth-child(odd) {
  /* background-color: #ccc;
  */
}
.test [class*="col"] {
  /* background-color: #E5E5E5;
  */
  color: black;
  padding: 1rem;
  /* margin-bottom:10px;
  */
}
.tight [class*="col"] {
  margin: 0;
  width: 25%;
  position: relative;
}
[class*="col"]{
   float:left;
/*    display:inline-block; */
}
[class*="col"]:last-child,[class*="col"].last {
  margin: 0;
}
.grid1 [class*="col"] {
  margin: 0 1% 0 0;
}
.grid0 [class*="col"] {
/*   float: left; */
  margin: 0;
}
.grid3 [class*="col"],[class*="col"] {
  margin: 0 3% 0 0;
  border-collapse: collapse;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  overflow: visible;
}
/*Based on 3% margin */
.grid3 .col1,.col1 {
  width: 5.578%;
}
.grid3 .col2,.col2 {
  width: 14.16%;
}
.grid3 .col3,.col3 {
  width: 22.75%;
}
.grid3 .col4,.col4 {
  width: 31.33%;
}
.grid3 .col5,.col5 {
  width: 39.5%;
}
.grid3 .col6,.col6 {
  width: 48.5%;
}
.grid3 .col7,.col7 {
  width: 57.5%;
}
.grid3 .col8,.col8 {
  width: 65.6%;
}
.grid3 .col9,.col9 {
  width: 74.2%;
}
.grid3 .col10,.col10 {
  width: 82.8%;
}
.grid3 .col11,.col11 {
  width: 91.4%;
}
.grid3 .col12,.col12 {
  width: 100%;
  margin: 0;
}
/*Based on 1% margin */
.grid1 .col1 {
  width: 7.33%
}
.grid1 .col2 {
  width: 15.65%
}
.grid1 .col3 {
  width: 24%
}
.grid1 .col4 {
  width: 32.3%
}
.grid1 .col5 {
  width: 37%
}
.grid1 .col6 {
  width: 49%
}
.grid1 .col7 {
  width: 61%
}
.grid1 .col8 {
  width: 65.7%
}
.grid1 .col9 {
  width: 74%
}
.grid1 .col10 {
  width: 82.3%
}
.grid1 .col11 {
  width: 90.6%
}
.grid1 .col12 {
  width: 98.8%;
  margin: 0
}
/*Based on 0% margin */
.grid0 .col1 {
  width: 8.33%;
}
.grid0 .col2 {
  width: 16.66%;
}
.grid0 .col3 {
  width: 24.99%;
}
.grid0 .col4 {
  width: 33.33%;
}
.grid0 .col5 {
  width: 39.5%
}
.grid0 .col6 {
  width: 50%;
}
.grid0 .col7 {
  width: 60.5%;
}
.grid0 .col8 {
  width: 66.6%;
}
.grid0 .col9 {
  width: 75%;
}
.grid0 .col10 {
  width: 83.3%;
}
.grid0 .col11 {
  width: 91.6%;
}
.grid0 .col12 {
  width: 99.8%;
  margin: 0
}
@media only screen and (max-width: 960px) {
  .mobile-two [class*="col"].last {
    margin: 0 3% 0 0;
  }
  .mobile-two .col4 {
    width: 47%;
  }
  .mobile-two.grid0 .col4 {
    width: 50%;
  }
  .mobile-two.grid1 .col4 {
    width: 49%;
  }

}
@media only screen and ( max-width: 640px) {
  [class*="col"],[class*="grid"] [class*="col"],.mobile-two [class*="col"] {
    float: none;
    width: 100%;
  }
}