[align="right"]{text-align: right;}
[align="center"]{text-align: center;}

/**** Elements with flex properties ****/
[flex] {display: flex;}
/* Aligns flex items horizontally along the main axis. */
[flex="center"]{justify-content: center;}
[flex="right"]{justify-content: flex-end;}
[flex="space-around"]{justify-content: space-around;}
[flex="space-between"]{justify-content: space-between;}
[flex="space-evenly"]{justify-content: space-evenly;}