{%- set site = configs -%}
{%- set default_logo = site.logo if site.logo else '' -%}
{%- set data = data if data else {"url": request.url, "status": "public"} -%}
{%- set nofollow = data and not data.slug -%}
{%- set canonical = data.canonical if data.canonical else request.raw_path -%}
<meta charset="utf-8">
<link rel="icon" href="/favicon.ico" type="image/x-icon">
<link rel="canonical" href="{{request.host}}{{request.url}}"" />
<title>{{ data.title|title|safe }} {{ site.name | safe }}</title>
{# <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0, user-scalable=1"> #}
<meta name="author" content="{{site.author|title or 'Pycasso CMS'}}" />
<meta property="description" content="{{ site.description }}" />
<meta property="og:locale" content="en_US" />
<meta property="og:type" content="{{data.type}}" />
<meta property="og:title" content="{{data.title|title}}" />
<meta property="og:url" content="{{request.host}}{{request.url}}" />
<meta property="og:site_name" content="{{ site.name }}" />
<meta property="article:publisher" content="{{ site.author|title }}" />
<meta name="viewport" content="width=device-width, initial-scale=1">
<meta name="robots" content="{{ 'noindex nofollow' if nofollow else 'all'}}"/>