/* Base font settings */
body {
  font-family: "Charter", Georgia, serif;
  font-size: 16px;
  line-height: 1.5;
  color: #000;
}
.courier-prime-regular {
  font-family: "Courier Prime", serif;
  font-weight: 400;
  font-style: normal;
}

.courier-prime-bold {
  font-family: "Courier Prime", serif;
  font-weight: 700;
  font-style: normal;
}

.courier-prime-regular-italic {
  font-family: "Courier Prime", serif;
  font-weight: 400;
  font-style: italic;
}

.courier-prime-bold-italic {
  font-family: "Courier Prime", serif;
  font-weight: 700;
  font-style: italic;
}
/*// <weight>: Use a value from 300 to 700*/
/*// <uniquifier>: Use a unique and descriptive class name*/

.hanken-grotesk-regular {
  font-family: "Hanken Grotesk", serif;
  font-optical-sizing: auto;
  font-weight: 600;
  font-style: normal;
}
.hanken-grotesk-bold {
  font-family: "Hanken Grotesk", serif;
  font-optical-sizing: auto;
  font-weight: 900;
  font-style: normal;
}

/* Typography scale */
h1 { font-size: 32px; line-height: 1.25; }
h2 { font-size: 24px; line-height: 1.3; }
h3 { font-size: 20px; line-height: 1.35; }
h4 { font-size: 18px; line-height: 1.4; }
p { font-size: 16px; line-height: 1.5; }
small, .meta-text { font-size: 14px; line-height: 1.4; }
a {
  color: inherit;
  outline: none;
  text-decoration: none;
}
a:focus {
  box-shadow: 0 0 0 .2em rgba(87, 100, 198, .15);
}
a,
a:focus,
a:hover,
a:active,
a.active {
  color: #000000;
  text-decoration: underline;
}

p {
  line-height: 1.4em;
  margin: 0 0 1em;
}

blockquote {
  border-left: .2em solid #f0f1f4;
  margin-left: 0;
  padding: 1em 2em;
}
blockquote p:last-child {
  margin-bottom: 0;
}
blockquote cite {
  color: #acb3c2;
}

ul ul,
ol ul,
ul ol,
ol ol {
  margin: 0.5em 0 1.5em 1em;
}

ul {
  list-style: disc inside;
}
ul ul {
  list-style-type: circle;
}
ol {
  list-style: decimal inside;
}
ol ol {
  list-style-type: lower-alpha;
}
dl dt {
  font-weight: bold;
}
dl dd {
  margin: .5em 0 1.5em 0;
}
strong{ font-weight: bold; }
code {
    background: #dedede;
    padding: 0 .534rem;
}