{# 
DO NOT EDIT THIS FILE IF YOU DONT KNOW WHAT YOU'RE DOING.
This is the master macros container.
Add a new macro and include the output as a separate file.
#}

{# BEGIN HEAD BLOCK #}
{% macro head(data=False) %}
{% include 'head.html' %}
{% endmacro %}
{# END HEAD BLOCK #}


{# BEGIN SHARETHIS BLOCK #}
{% macro sharethis(settings=False, data=False) %}
{% include 'ui_components/sharethis.html' %}
{% endmacro %}
{# END SHARETHIS BLOCK #}


{# BEGIN MULTIMEDIA #}
{% macro media(settings=False, data=False) %}
{% include 'multimedia.html' %}
{% endmacro %}
{# END MULTIMEDIA BLOCK #}





{# BEGIN GALLERY #}
{% macro gallery(settings=False,data=False) %}
{% include 'ui_components/gallery.html' %}
{% endmacro %}
{# END GALLERY #}
